@@ -207,8 +207,6 @@ class PortSet(app_manager.RyuApp):
             tunnel_key = self.tunnels.get_key(network_id)
         except tunnels.TunnelKeyNotFound:
             return
-        for (dpid, port_no) in self.nw.list_ports(network_id):
-            self._vm_port_mac_handler(dpid, port_no, network_id, False)
         self._tunnel_key_del(tunnel_key)
 
     @handler.set_ev_cls(network.EventNetworkPort)
@@ -223,15 +221,16 @@ class PortSet(app_manager.RyuApp):
 
     @handler.set_ev_cls(tunnels.EventTunnelKeyAdd)
     def tunnel_key_add_handler(self, ev):
-        for (dpid, port_no) in self.nw.list_ports(ev.network_id):
-            self._vm_port_mac_handler(dpid, port_no, ev.network_id, True)
+        network_id = ev.network_id
+        for (dpid, port_no) in self.nw.list_ports_noraise(network_id):
+            self._vm_port_mac_handler(dpid, port_no, network_id, True)
 
     @handler.set_ev_cls(tunnels.EventTunnelKeyDel)
     def tunnel_key_del_handler(self, ev):
         network_id = ev.network_id
-        for (dpid, port_no) in self.nw.list_ports(network_id):
+        for (dpid, port_no) in self.nw.list_ports_noraise(network_id):
             self._vm_port_mac_handler(dpid, port_no, network_id, False)
-        if self.nw.has_networks(network_id):
+        if self.nw.has_network(network_id):
             self._tunnel_key_del(ev.tunnel_key)
 
     @handler.set_ev_cls(tunnels.EventTunnelPort)
@@ -395,7 +394,7 @@ class GRETunnel(app_manager.RyuApp):
         self.port_set = PortSet(**kwargs)
         map(lambda ev_cls: self.port_set.register_observer(ev_cls, self.name),
             [dpset.EventDP, PortSet.EventTunnelKeyDel, PortSet.EventVMPort,
-            PortSet.EventTunnelPort, ofp_event.EventOFPPacketIn])
+             PortSet.EventTunnelPort, ofp_event.EventOFPPacketIn])
 
     # TODO: track active vm/tunnel ports
 
@@ -958,5 +957,5 @@ class GRETunnel(app_manager.RyuApp):
         # for debug
         msg = ev.msg
         self.logger.debug('packet in ev %s msg %s', ev, ev.msg)
-        if msg.buffer_id != 0xffffffff:  # TODO:XXX use constant instead of -1
+        if msg.buffer_id != msg.datapath.ofproto.OFP_NO_BUFFER:
             msg.datapath.send_packet_out(msg.buffer_id, msg.in_port, [])
