@@ -14,8 +14,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import logging
-
 from ryu.app import (conf_switch_key,
                      rest_nw_id)
 from ryu.base import app_manager
@@ -201,7 +199,7 @@ class SimpleVLAN(app_manager.RyuApp):
     def tunnel_key_add_handler(self, ev):
         self.logger.debug('tunnel_add %s', ev)
         tunnel_key = ev.tunnel_key
-        for (dpid, port_no) in self.nw.list_ports(ev.network_id):
+        for (dpid, port_no) in self.nw.list_ports_noraise(ev.network_id):
             dp = self.dpset.get(dpid)
             if dp is None:
                 continue
