@@ -43,17 +43,29 @@ class ethernet(packet_base.PacketBase):
         self.dst = dst
         self.src = src
         self.ethertype = ethertype
-        self.length = ethernet._MIN_LEN
 
     @classmethod
     def parser(cls, buf):
         dst, src, ethertype = struct.unpack_from(cls._PACK_STR, buf)
-        return cls(dst, src, ethertype), ethernet.get_packet_type(ethertype)
+        return (cls(dst, src, ethertype), ethernet.get_packet_type(ethertype),
+                buf[ethernet._MIN_LEN:])
 
     def serialize(self, payload, prev):
         return struct.pack(ethernet._PACK_STR, self.dst, self.src,
                            self.ethertype)
 
+    @classmethod
+    def get_packet_type(cls, type_):
+        """Override method for the ethernet IEEE802.3 Length/Type
+        field (self.ethertype).
+
+        If the value of Length/Type field is less than or equal to
+        1500 decimal(05DC hexadecimal), it means Length interpretation
+        and be passed to the LLC sublayer."""
+        if type_ <= ether.ETH_TYPE_IEEE802_3:
+            type_ = ether.ETH_TYPE_IEEE802_3
+        return cls._TYPES.get(type_)
+
 
 # copy vlan _TYPES
 ethernet._TYPES = vlan.vlan._TYPES
