@@ -106,7 +106,7 @@ class icmpv6(packet_base.PacketBase):
             else:
                 msg.data = buf[offset:]
 
-        return msg, None
+        return msg, None, None
 
     def serialize(self, payload, prev):
         hdr = bytearray(struct.pack(icmpv6._PACK_STR, self.type_,
