@@ -21,11 +21,12 @@ from . import icmp
 from . import udp
 from . import tcp
 from ryu.ofproto import inet
+from ryu.lib import ip
 
 
 IPV4_ADDRESS_PACK_STR = '!I'
 IPV4_ADDRESS_LEN = struct.calcsize(IPV4_ADDRESS_PACK_STR)
-IPV4_PSEUDO_HEADER_PACK_STR = '!II2xHH'
+IPV4_PSEUDO_HEADER_PACK_STR = '!4s4s2xHH'
 
 
 class ipv4(packet_base.PacketBase):
@@ -61,12 +62,15 @@ class ipv4(packet_base.PacketBase):
     ============== ====================
     """
 
-    _PACK_STR = '!BBHHHBBHII'
+    _PACK_STR = '!BBHHHBBH4s4s'
     _MIN_LEN = struct.calcsize(_PACK_STR)
 
-    def __init__(self, version, header_length, tos, total_length,
-                 identification, flags, offset, ttl, proto, csum,
-                 src, dst, option=None):
+    def __init__(self, version=4, header_length=5, tos=0,
+                 total_length=0, identification=0, flags=0,
+                 offset=0, ttl=255, proto=0, csum=0,
+                 src=ip.ipv4_to_bin('0.0.0.0'),
+                 dst=ip.ipv4_to_bin('0.0.0.0'),
+                 option=None):
         super(ipv4, self).__init__()
         self.version = version
         self.header_length = header_length
@@ -80,9 +84,11 @@ class ipv4(packet_base.PacketBase):
         self.csum = csum
         self.src = src
         self.dst = dst
-        self.length = header_length * 4
         self.option = option
 
+    def __len__(self):
+        return self.header_length * 4
+
     @classmethod
     def parser(cls, buf):
         (version, tos, total_length, identification, flags, ttl, proto, csum,
@@ -91,16 +97,19 @@ class ipv4(packet_base.PacketBase):
         version = version >> 4
         offset = flags & ((1 << 13) - 1)
         flags = flags >> 13
+        length = header_length * 4
+        if length > ipv4._MIN_LEN:
+            option = buf[ipv4._MIN_LEN:length]
+        else:
+            option = None
         msg = cls(version, header_length, tos, total_length, identification,
-                  flags, offset, ttl, proto, csum, src, dst)
+                  flags, offset, ttl, proto, csum, src, dst, option)
 
-        if msg.length > ipv4._MIN_LEN:
-            msg.option = buf[ipv4._MIN_LEN:msg.length]
-
-        return msg, ipv4.get_packet_type(proto)
+        return msg, ipv4.get_packet_type(proto), buf[length:total_length]
 
     def serialize(self, payload, prev):
-        hdr = bytearray(self.header_length * 4)
+        length = len(self)
+        hdr = bytearray(length)
         version = self.version << 4 | self.header_length
         flags = self.flags << 13 | self.offset
         if self.total_length == 0:
@@ -110,7 +119,7 @@ class ipv4(packet_base.PacketBase):
                          self.ttl, self.proto, 0, self.src, self.dst)
 
         if self.option:
-            assert (self.length - ipv4._MIN_LEN) >= len(self.option)
+            assert (length - ipv4._MIN_LEN) >= len(self.option)
             hdr[ipv4._MIN_LEN:ipv4._MIN_LEN + len(self.option)] = self.option
 
         self.csum = packet_utils.checksum(hdr)
