@@ -59,36 +59,42 @@ class tcp(packet_base.PacketBase):
         self.window_size = window_size
         self.csum = csum
         self.urgent = urgent
-        self.length = self.offset * 4
         self.option = option
 
+    def __len__(self):
+        return self.offset * 4
+
     @classmethod
     def parser(cls, buf):
         (src_port, dst_port, seq, ack, offset, bits, window_size,
          csum, urgent) = struct.unpack_from(cls._PACK_STR, buf)
         offset = offset >> 4
         bits = bits & 0x3f
+        length = offset * 4
+        if length > tcp._MIN_LEN:
+            option = buf[tcp._MIN_LEN:length]
+        else:
+            option = None
         msg = cls(src_port, dst_port, seq, ack, offset, bits,
-                  window_size, csum, urgent)
+                  window_size, csum, urgent, option)
 
-        if msg.length > tcp._MIN_LEN:
-            msg.option = buf[tcp._MIN_LEN:msg.length]
-
-        return msg, None
+        return msg, None, buf[length:]
 
     def serialize(self, payload, prev):
-        h = bytearray(self.length)
+        length = len(self)
+        h = bytearray(length)
         offset = self.offset << 4
         struct.pack_into(tcp._PACK_STR, h, 0, self.src_port, self.dst_port,
                          self.seq, self.ack, offset, self.bits,
                          self.window_size, self.csum, self.urgent)
 
         if self.option:
-            assert (self.length - tcp._MIN_LEN) >= len(self.option)
+            assert (length - tcp._MIN_LEN) >= len(self.option)
             h[tcp._MIN_LEN:tcp._MIN_LEN + len(self.option)] = self.option
 
         if self.csum == 0:
-            length = self.length + len(payload)
-            self.csum = packet_utils.checksum_ip(prev, length, h + payload)
+            total_length = length + len(payload)
+            self.csum = packet_utils.checksum_ip(prev, total_length,
+                                                 h + payload)
             struct.pack_into('!H', h, 16, self.csum)
         return h
