@@ -19,6 +19,8 @@ from . import arp
 from . import ipv4
 from . import ipv6
 from . import lldp
+from . import slow
+from . import llc
 from ryu.ofproto import ether
 from ryu.ofproto.ofproto_parser import msg_pack_into
 
@@ -43,13 +45,24 @@ class vlan(packet_base.PacketBase):
     _PACK_STR = "!HH"
     _MIN_LEN = struct.calcsize(_PACK_STR)
 
-    def __init__(self, pcp, cfi, vid, ethertype):
+    def __init__(self, pcp=0, cfi=0, vid=0, ethertype=ether.ETH_TYPE_IP):
         super(vlan, self).__init__()
         self.pcp = pcp
         self.cfi = cfi
         self.vid = vid
         self.ethertype = ethertype
-        self.length = vlan._MIN_LEN
+
+    @classmethod
+    def get_packet_type(cls, type_):
+        """Override method for the Length/Type field (self.ethertype).
+        The Length/Type field means Length or Type interpretation,
+        same as ethernet IEEE802.3.
+        If the value of Length/Type field is less than or equal to
+        1500 decimal(05DC hexadecimal), it means Length interpretation
+        and be passed to the LLC sublayer."""
+        if type_ <= ether.ETH_TYPE_IEEE802_3:
+            type_ = ether.ETH_TYPE_IEEE802_3
+        return cls._TYPES.get(type_)
 
     @classmethod
     def parser(cls, buf):
@@ -57,7 +70,8 @@ class vlan(packet_base.PacketBase):
         pcp = tci >> 13
         cfi = (tci >> 12) & 1
         vid = tci & ((1 << 12) - 1)
-        return cls(pcp, cfi, vid, ethertype), vlan.get_packet_type(ethertype)
+        return (cls(pcp, cfi, vid, ethertype),
+                vlan.get_packet_type(ethertype), buf[vlan._MIN_LEN:])
 
     def serialize(self, payload, prev):
         tci = self.pcp << 13 | self.cfi << 12 | self.vid
@@ -67,3 +81,5 @@ vlan.register_packet_type(arp.arp, ether.ETH_TYPE_ARP)
 vlan.register_packet_type(ipv4.ipv4, ether.ETH_TYPE_IP)
 vlan.register_packet_type(ipv6.ipv6, ether.ETH_TYPE_IPV6)
 vlan.register_packet_type(lldp.lldp, ether.ETH_TYPE_LLDP)
+vlan.register_packet_type(slow.slow, ether.ETH_TYPE_SLOW)
+vlan.register_packet_type(llc.llc, ether.ETH_TYPE_IEEE802_3)
