@@ -88,7 +88,7 @@ VRRP_IPV4_SRC_MAC_ADDRESS = netaddr.EUI(VRRP_IPV4_SRC_MAC_ADDRESS_STR).packed
 VRRP_IPV4_DST_MAC_ADDRESS_STR = '01:00:5E:00:00:12'
 VRRP_IPV4_DST_MAC_ADDRESS = netaddr.EUI(VRRP_IPV4_DST_MAC_ADDRESS_STR).packed
 VRRP_IPV4_DST_ADDRESS_STR = '224.0.0.18'
-VRRP_IPV4_DST_ADDRESS = netaddr.IPAddress(VRRP_IPV4_DST_ADDRESS_STR).value
+VRRP_IPV4_DST_ADDRESS = netaddr.IPAddress(VRRP_IPV4_DST_ADDRESS_STR).packed
 VRRP_IPV4_TTL = 255
 
 
@@ -166,10 +166,9 @@ VRRP_V2_MAX_ADVER_INT_MAX = 0xff
 
 
 def is_ipv6(ip_address):
-    if type(ip_address) == int:
-        return False
-
     assert type(ip_address) == str
+    if len(ip_address) == 4:
+        return False
     assert len(ip_address) == 16
     return True
 
@@ -203,7 +202,7 @@ class vrrp(packet_base.PacketBase):
     """
 
     _VERSION_PACK_STR = '!B'
-    _IPV4_ADDRESS_PACK_STR_RAW = 'I'
+    _IPV4_ADDRESS_PACK_STR_RAW = '4s'
     _IPV4_ADDRESS_PACK_STR = '!' + _IPV4_ADDRESS_PACK_STR_RAW
     _IPV4_ADDRESS_LEN = struct.calcsize(_IPV4_ADDRESS_PACK_STR)
     _IPV6_ADDRESS_LEN = 16
@@ -270,7 +269,6 @@ class vrrp(packet_base.PacketBase):
         self.auth_data = auth_data
 
         self._is_ipv6 = is_ipv6(self.ip_addresses[0])
-        self.length = len(self)
         self.identification = 0         # used for ipv4 identification
 
     def checksum_ok(self, ipvx, vrrp_buf):
@@ -337,7 +335,7 @@ class vrrp(packet_base.PacketBase):
         if self.is_ipv6:
             traffic_class = 0xc0        # set tos to internetwork control
             flow_label = 0
-            payload_length = ipv6.ipv6._MIN_LEN + self.length   # XXX _MIN_LEN
+            payload_length = ipv6.ipv6._MIN_LEN + len(self)     # XXX _MIN_LEN
             e = ethernet.ethernet(VRRP_IPV6_DST_MAC_ADDRESS,
                                   vrrp_ipv6_src_mac_address(self.vrid),
                                   ether.ETH_TYPE_IPV6)
@@ -456,8 +454,9 @@ class vrrpv2(vrrp):
         offset += struct.calcsize(ip_addresses_pack_str)
         auth_data = struct.unpack_from(cls._AUTH_DATA_PACK_STR, buf, offset)
 
-        return cls(version, type_, vrid, priority, count_ip, adver_int,
-                   checksum, ip_addresses, auth_type, auth_data), None
+        msg = cls(version, type_, vrid, priority, count_ip, adver_int,
+                  checksum, ip_addresses, auth_type, auth_data)
+        return msg, None, buf[len(msg):]
 
     @staticmethod
     def serialize_static(vrrp_, prev):
@@ -534,7 +533,7 @@ class vrrpv3(vrrp):
         # http://www.ietf.org/mail-archive/web/vrrp/current/msg01473.html
         # if not self.is_ipv6:
         #     return packet_utils.checksum(vrrp_buf) == 0
-        return packet_utils.checksum_ip(ipvx, self.length, vrrp_buf) == 0
+        return packet_utils.checksum_ip(ipvx, len(self), vrrp_buf) == 0
 
     @staticmethod
     def create(type_, vrid, priority, max_adver_int, ip_addresses):
@@ -573,13 +572,15 @@ class vrrpv3(vrrp):
                     address_len, count_ip))
 
         ip_addresses = struct.unpack_from(pack_str, buf, offset)
-        return cls(version, type_, vrid, priority,
-                   count_ip, max_adver_int, checksum, ip_addresses), None
+        msg = cls(version, type_, vrid, priority,
+                  count_ip, max_adver_int, checksum, ip_addresses)
+        return msg, None, buf[len(msg):]
 
     @staticmethod
     def serialize_static(vrrp_, prev):
         if isinstance(prev, ipv4.ipv4):
-            assert type(vrrp_.ip_addresses[0]) == int
+            assert type(vrrp_.ip_addresses[0]) == str
+            assert len(vrrp_.ip_addresses[0]) == 4
             ip_address_pack_raw = vrrpv3._IPV4_ADDRESS_PACK_STR_RAW
         elif isinstance(prev, ipv6.ipv6):
             assert type(vrrp_.ip_addresses[0]) == str
