@@ -74,7 +74,7 @@ class Test_tcp(unittest.TestCase):
         eq_(self.option, self.t.option)
 
     def test_parser(self):
-        r1, r2 = self.t.parser(self.buf)
+        r1, r2, _ = self.t.parser(self.buf)
 
         eq_(self.src_port, r1.src_port)
         eq_(self.dst_port, r1.dst_port)
@@ -92,8 +92,8 @@ class Test_tcp(unittest.TestCase):
         offset = 5
         csum = 0
 
-        src_ip = int(netaddr.IPAddress('192.168.10.1'))
-        dst_ip = int(netaddr.IPAddress('192.168.100.1'))
+        src_ip = netaddr.IPAddress('192.168.10.1').packed
+        dst_ip = netaddr.IPAddress('192.168.100.1').packed
         prev = ipv4(4, 5, 0, 0, 0, 0, 0, 64,
                     inet.IPPROTO_TCP, 0, src_ip, dst_ip)
 
@@ -112,7 +112,7 @@ class Test_tcp(unittest.TestCase):
         eq_(res[8], self.urgent)
 
         # checksum
-        ph = struct.pack('!IIBBH', src_ip, dst_ip, 0, 6, offset * 4)
+        ph = struct.pack('!4s4sBBH', src_ip, dst_ip, 0, 6, offset * 4)
         d = ph + buf + bytearray()
         s = packet_utils.checksum(d)
         eq_(0, s)
@@ -122,8 +122,8 @@ class Test_tcp(unittest.TestCase):
         csum = 0
         option = '\x01\x02'
 
-        src_ip = int(netaddr.IPAddress('192.168.10.1'))
-        dst_ip = int(netaddr.IPAddress('192.168.100.1'))
+        src_ip = netaddr.IPAddress('192.168.10.1').packed
+        dst_ip = netaddr.IPAddress('192.168.100.1').packed
         prev = ipv4(4, 5, 0, 0, 0, 0, 0, 64,
                     inet.IPPROTO_TCP, 0, src_ip, dst_ip)
 
