@@ -631,13 +631,14 @@ class Switches(app_manager.RyuApp):
 
     @staticmethod
     def _drop_packet(msg):
-        if msg.buffer_id == 0xffffffff:
-            return  # TODO:use constant instead of -1
+        buffer_id = msg.buffer_id
+        if buffer_id == msg.datapath.ofproto.OFP_NO_BUFFER:
+            return
 
         dp = msg.datapath
         # TODO:XXX
         if dp.ofproto.OFP_VERSION == ofproto_v1_0.OFP_VERSION:
-            dp.send_packet_out(dp.id, msg.in_port, [])
+            dp.send_packet_out(buffer_id, msg.in_port, [])
         else:
             LOG.error('cannot drop_packet. unsupported version. %x',
                       dp.ofproto.OFP_VERSION)
@@ -654,47 +655,46 @@ class Switches(app_manager.RyuApp):
             # This handler can receive all the packtes which can be
             # not-LLDP packet. Ignore it silently
             return
-        else:
-            dst_dpid = msg.datapath.id
-            dst_port_no = msg.in_port
 
-            src = self._get_port(src_dpid, src_port_no)
-            if not src or src.dpid == dst_dpid:
-                return
+        dst_dpid = msg.datapath.id
+        dst_port_no = msg.in_port
 
-            dst = self._get_port(dst_dpid, dst_port_no)
-            if not dst:
-                return
+        src = self._get_port(src_dpid, src_port_no)
+        if not src or src.dpid == dst_dpid:
+            return
+        try:
+            self.ports.lldp_received(src)
+        except KeyError:
+            # There are races between EventOFPPacketIn and
+            # EventDPPortAdd. So packet-in event can happend before
+            # port add event. In that case key error can happend.
+            # LOG.debug('lldp_received: KeyError %s', e)
+            pass
 
-            old_peer = self.links.get_peer(src)
-            # LOG.debug("Packet-In")
-            # LOG.debug("  src=%s", src)
-            # LOG.debug("  dst=%s", dst)
-            # LOG.debug("  old_peer=%s", old_peer)
-            if old_peer and old_peer != dst:
-                old_link = Link(src, old_peer)
-                self.send_event_to_observers(event.EventLinkDelete(old_link))
+        dst = self._get_port(dst_dpid, dst_port_no)
+        if not dst:
+            return
 
-            link = Link(src, dst)
-            if not link in self.links:
-                self.send_event_to_observers(event.EventLinkAdd(link))
+        old_peer = self.links.get_peer(src)
+        # LOG.debug("Packet-In")
+        # LOG.debug("  src=%s", src)
+        # LOG.debug("  dst=%s", dst)
+        # LOG.debug("  old_peer=%s", old_peer)
+        if old_peer and old_peer != dst:
+            old_link = Link(src, old_peer)
+            self.send_event_to_observers(event.EventLinkDelete(old_link))
 
-            if not self.links.update_link(src, dst):
-                # reverse link is not detected yet.
-                # So schedule the check early because it's very likely it's up
-                try:
-                    self.ports.lldp_received(dst)
-                except KeyError as e:
-                    # There are races between EventOFPPacketIn and
-                    # EventDPPortAdd. So packet-in event can happend before
-                    # port add event. In that case key error can happend.
-                    # LOG.debug('lldp_received: KeyError %s', e)
-                    pass
-                else:
-                    self.ports.move_front(dst)
-                    self.lldp_event.set()
-            if self.explicit_drop:
-                self._drop_packet(msg)
+        link = Link(src, dst)
+        if not link in self.links:
+            self.send_event_to_observers(event.EventLinkAdd(link))
+
+        if not self.links.update_link(src, dst):
+            # reverse link is not detected yet.
+            # So schedule the check early because it's very likely it's up
+            self.ports.move_front(dst)
+            self.lldp_event.set()
+        if self.explicit_drop:
+            self._drop_packet(msg)
 
     def send_lldp_packet(self, port):
         try:
